=== added directory 'bzrlib/repofmt'
=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2007-01-17 15:37:08 +0000
+++ bzrlib/repository.py	2007-01-24 11:20:17 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -37,6 +37,7 @@
     lockable_files,
     lockdir,
     osutils,
+    registry,
     revision as _mod_revision,
     symbol_versioning,
     transactions,
@@ -1135,6 +1136,15 @@
                                  committer, revprops, revision_id)
 
 
+class RepositoryFormatRegistry(registry.Registry):
+    """Registry of RepositoryFormats.
+    """
+    
+
+format_registry = RepositoryFormatRegistry()
+"""Registry of formats, indexed by their identifying format string."""
+
+
 class RepositoryFormat(object):
     """A repository format.
 
@@ -1159,35 +1169,55 @@
     parameterisation.
     """
 
-    _default_format = None
-    """The default format used for new repositories."""
-
-    _formats = {}
-    """The known formats."""
-
     def __str__(self):
         return "<%s>" % self.__class__.__name__
 
     @classmethod
     def find_format(klass, a_bzrdir):
-        """Return the format for the repository object in a_bzrdir."""
+        """Return the format for the repository object in a_bzrdir.
+        
+        This is used by bzr native formats that have a "format" file in
+        the repository.  Other methods may be used by different types of 
+        control directory.
+        """
         try:
             transport = a_bzrdir.get_repository_transport(None)
             format_string = transport.get("format").read()
-            return klass._formats[format_string]
+            return format_registry.get(format_string)
         except errors.NoSuchFile:
             raise errors.NoRepositoryPresent(a_bzrdir)
         except KeyError:
             raise errors.UnknownFormatError(format=format_string)
 
-    def _get_control_store(self, repo_transport, control_files):
-        """Return the control store for this repository."""
-        raise NotImplementedError(self._get_control_store)
+    @classmethod
+    @deprecated_method(symbol_versioning.zero_fourteen)
+    def set_default_format(klass, format):
+        klass._set_default_format(format)
+
+    @classmethod
+    def _set_default_format(klass, format):
+        """Set the default format for new Repository creation.
+
+        The format must already be registered.
+        """
+        format_registry.default_key = format.get_format_string()
+
+    @classmethod
+    def register_format(klass, format):
+        format_registry.register(format.get_format_string(), format)
+
+    @classmethod
+    def unregister_format(klass, format):
+        format_registry.remove(format.get_format_string())
     
     @classmethod
     def get_default_format(klass):
         """Return the current default format."""
-        return klass._default_format
+        return format_registry.get(format_registry.default_key)
+
+    def _get_control_store(self, repo_transport, control_files):
+        """Return the control store for this repository."""
+        raise NotImplementedError(self._get_control_store)
 
     def get_format_string(self):
         """Return the ASCII format string that identifies this format.
@@ -1275,24 +1305,6 @@
         """
         raise NotImplementedError(self.open)
 
-    @classmethod
-    def register_format(klass, format):
-        klass._formats[format.get_format_string()] = format
-
-    @classmethod
-    @deprecated_method(symbol_versioning.zero_fourteen)
-    def set_default_format(klass, format):
-        klass._set_default_format(format)
-
-    @classmethod
-    def _set_default_format(klass, format):
-        klass._default_format = format
-
-    @classmethod
-    def unregister_format(klass, format):
-        assert klass._formats[format.get_format_string()] is format
-        del klass._formats[format.get_format_string()]
-
 
 class PreSplitOutRepositoryFormat(RepositoryFormat):
     """Base class for the pre split out repository formats."""
@@ -1807,7 +1819,8 @@
 # formats which have no format string are not discoverable
 # and not independently creatable, so are not registered.
 RepositoryFormat.register_format(RepositoryFormat7())
-# KEEP in sync with bzrdir.format_registry default
+# KEEP in sync with bzrdir.format_registry default, which controls the overall
+# default control directory format
 _default_format = RepositoryFormatKnit1()
 RepositoryFormat.register_format(_default_format)
 RepositoryFormat.register_format(RepositoryFormatKnit2())

=== modified file 'bzrlib/tests/repository_implementations/__init__.py'
--- bzrlib/tests/repository_implementations/__init__.py	2006-10-11 23:08:27 +0000
+++ bzrlib/tests/repository_implementations/__init__.py	2007-01-25 00:35:22 +0000
@@ -1,6 +1,6 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2007 Canonical Ltd
 # Authors: Robert Collins <robert.collins@canonical.com>
-# -*- coding: utf-8 -*-
+#          and others
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,8 +24,10 @@
 rather than in tests/branch_implementations/*.py.
 """
 
+from bzrlib import (
+    repository,
+    )
 from bzrlib.repository import (_legacy_formats,
-                               RepositoryFormat,
                                RepositoryTestProviderAdapter,
                                )
                             
@@ -47,13 +49,14 @@
         'bzrlib.tests.repository_implementations.test_repository',
         'bzrlib.tests.repository_implementations.test_revision',
         ]
+    all_formats = [v for (k, v) in repository.format_registry.iteritems()]
+    all_formats.extend(_legacy_formats)
     adapter = RepositoryTestProviderAdapter(
         default_transport,
         # None here will cause a readonly decorator to be created
         # by the TestCaseWithTransport.get_readonly_transport method.
         None,
-        [(format, format._matchingbzrdir) for format in 
-         RepositoryFormat._formats.values() + _legacy_formats])
+        [(format, format._matchingbzrdir) for format in all_formats])
     loader = TestLoader()
     adapt_modules(test_repository_implementations, adapter, loader, result)
     return result

=== modified file 'bzrlib/tests/test_repository.py'
--- bzrlib/tests/test_repository.py	2007-01-05 06:32:57 +0000
+++ bzrlib/tests/test_repository.py	2007-01-24 07:12:09 +0000
@@ -34,33 +34,40 @@
                            UnknownFormatError,
                            UnsupportedFormatError,
                            )
-import bzrlib.repository as repository
+from bzrlib.repository import RepositoryFormat
 from bzrlib.tests import TestCase, TestCaseWithTransport
 from bzrlib.transport import get_transport
 from bzrlib.transport.memory import MemoryServer
-from bzrlib import upgrade, workingtree
+from bzrlib import (
+    repository,
+    upgrade,
+    workingtree,
+    )
 
 
 class TestDefaultFormat(TestCase):
 
     def test_get_set_default_format(self):
-        private_default = repository._default_format.__class__
-        old_format = repository.RepositoryFormat.get_default_format()
-        self.assertTrue(isinstance(old_format, private_default))
-        self.applyDeprecated(symbol_versioning.zero_fourteen, 
-            repository.RepositoryFormat.set_default_format, 
-            SampleRepositoryFormat())
-        # creating a repository should now create an instrumented dir.
+        old_format = RepositoryFormat.get_default_format()
+        test_format = SampleRepositoryFormat()
+        RepositoryFormat.register_format(test_format)
         try:
-            # the default branch format is used by the meta dir format
-            # which is not the default bzrdir format at this point
-            dir = bzrdir.BzrDirMetaFormat1().initialize('memory:///')
-            result = dir.create_repository()
-            self.assertEqual(result, 'A bzr repository dir')
-        finally:
             self.applyDeprecated(symbol_versioning.zero_fourteen, 
-                repository.RepositoryFormat.set_default_format, old_format)
-        self.assertEqual(old_format, repository.RepositoryFormat.get_default_format())
+                RepositoryFormat.set_default_format, 
+                test_format)
+            # creating a repository should now create an instrumented dir.
+            try:
+                # the default branch format is used by the meta dir format
+                # which is not the default bzrdir format at this point
+                dir = bzrdir.BzrDirMetaFormat1().initialize('memory:///')
+                result = dir.create_repository()
+                self.assertEqual(result, 'A bzr repository dir')
+            finally:
+                self.applyDeprecated(symbol_versioning.zero_fourteen,
+                    RepositoryFormat.set_default_format, old_format)
+        finally:
+            RepositoryFormat.unregister_format(test_format)
+        self.assertEqual(old_format, RepositoryFormat.get_default_format())
 
 
 class SampleRepositoryFormat(repository.RepositoryFormat):

